<?php
namespace FormAlert;

use Exception;
use WP_REST_Request;
use WP_REST_Response;

if (!defined('ABSPATH')) {
  exit;
}
class Class_api
{
  private $db;
  private $rest_api_base = "form-alert/v1";

  public function __construct(Class_database $db)
  {
    add_action('rest_api_init', [$this, 'register_routes']);

    $this->db = $db;
  }

  public function register_routes()
  {
    register_rest_route($this->rest_api_base, "/keys", [
      "methods" => "POST",
      "callback" => [$this, "create_api_key"],
      "permission_callback" => function () {
        return current_user_can('manage_options');
      },
    ]);

    register_rest_route($this->rest_api_base, "/keys/(?P<id>\d+)", [
      "methods" => "DELETE",
      "callback" => [$this, "delete_api_key"],
      "permission_callback" => function () {
        return current_user_can('manage_options');
      },
    ]);

    register_rest_route(
      $this->rest_api_base,
      "/keys",
      [
        "methods" => "GET",
        "callback" => [$this, "get_api_keys"],
        "permission_callback" => function () {
          return current_user_can('manage_options');
        }
      ]
    );
  }

  public function create_api_key(WP_REST_Request $req)
  {
    global $wpdb;

    $nonce = $req->get_header('X-WP-Nonce');

    if (!wp_verify_nonce($nonce, 'wp_rest')) {
      return new WP_REST_Response('Invalid nonce', 403);
    }

    $body = json_decode($req->get_body());

    if (!$body) {
      return new WP_REST_Response("Invalid request body", 400, [
        "Content-Type" => "text/plain",
      ]);
    }

    if (!isset($body->api_key) || !isset($body->name)) {
      return new WP_REST_Response("Invalid request body", 400, [
        "Content-Type" => "text/plain",
      ]);
    }

    $api_key = sanitize_text_field($body->api_key);
    $name = sanitize_text_field($body->name);

    if (empty($api_key) || empty($name)) {
      return new WP_REST_Response("Invalid request body", 400, [
        "Content-Type" => "text/plain",
      ]);
    }

    try {
      $exists = $wpdb->get_var(
        $wpdb->prepare(
          "SELECT * FROM {$this->db->table_name()} WHERE api_key = %s",
          $api_key
        )
      );

      if ($exists) {
        return new WP_REST_Response("API Key already exists", 400, [
          "Content-Type" => "text/plain",
        ]);
      }

      $wpdb->query(
        $wpdb->prepare(
          "INSERT INTO {$this->db->table_name()} (api_key, name) 
            VALUES (%s, %s)",
          $api_key,
          $name
        )
      );
      if ($wpdb->last_error) {
        throw new Exception("Something went wrong");
      }
      return new WP_REST_Response("API Key added successfully!", 200, [
        "Content-Type" => "text/plain",
      ]);
    } catch (Exception $e) {
      return new WP_REST_Response($e->getMessage(), 500, [
        "Content-Type" => "text/plain",
      ]);
    }
  }

  public function delete_api_key(WP_REST_Request $req)
  {
    global $wpdb;

    $nonce = $req->get_header('X-WP-Nonce');

    if (!wp_verify_nonce($nonce, 'wp_rest')) {
      return new WP_REST_Response('Invalid nonce', 403);
    }

    $id = $req->get_param('id');

    if (!$id) {
      return new WP_REST_Response('Invalid request body', 400, [
        "Content-Type" => "text/plain",
      ]);
    }

    try {
      $wpdb->query(
        $wpdb->prepare(
          "DELETE FROM {$this->db->table_name()} WHERE id = %d",
          $id
        )
      );
      if ($wpdb->last_error) {
        throw new Exception("Something went wrong");
      }
      return new WP_REST_Response("API Key deleted successfully!", 200, [
        "Content-Type" => "text/plain",
      ]);
    } catch (Exception $e) {
      return new WP_REST_Response($e->getMessage(), 500, [
        "Content-Type" => "text/plain",
      ]);
    }
  }

  public function get_api_keys(WP_REST_Request $req)
  {
    global $wpdb;

    $nonce = $req->get_header('X-WP-Nonce');

    if (!wp_verify_nonce($nonce, 'wp_rest')) {
      return new WP_REST_Response('Invalid nonce', 403);
    }

    try {
      $keys = $wpdb->get_results(
        $wpdb->prepare(
          "SELECT * FROM {$this->db->table_name()}"
        )
      );

      if ($keys) {
        return new WP_REST_Response($keys, 200, [
          "Content-Type" => "application/json",
        ]);
      }

      return new WP_REST_Response('Something went wrong', 500, [
        "Content-Type" => "text/plain",
      ]);
    } catch (Exception $e) {
      return new WP_REST_Response($e->getMessage(), 500, [
        "Content-Type" => "text/plain",
      ]);
    }
  }
}